package br.pucrs.ages.victims.auth.service

import br.pucrs.ages.victims.auth.domain.User
import br.pucrs.ages.victims.auth.exception.ExistingResourceException
import br.pucrs.ages.victims.auth.exception.ValidationException
import br.pucrs.ages.victims.auth.repository.UserRepository
import br.pucrs.ages.victims.auth.validator.UserValidator
import br.pucrs.ages.victims.auth.web.representation.request.RegisterRequestModel
import io.mockk.*
import org.junit.jupiter.api.Assertions.assertNotNull
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.assertThrows
import java.util.*

class UserServiceTest {

    private val userRepository = mockk<UserRepository>()
    private val userValidator = mockk<UserValidator>()

    private val userService = UserService(userRepository, userValidator)

    @Test
    fun `should create user when valid payload is provided`() {
        val payload = RegisterRequestModel("teste", "12345678911", "OAKODOab@1", "OAKODOab@1")
        val mockedUser = anUser()

        every { userValidator.validatePayload(any()) } just runs
        every { userRepository.findByCpf(any()) } returns Optional.empty()
        every { userRepository.updatePasswordById(any(), any()) } just runs
        every { userRepository.save(any()) } returns mockedUser
        every { userRepository.findByEmail(any()) } returns Optional.empty()

        val user = userService.create(payload)

        assertNotNull(user.id)
        verify { userRepository.save(any()) }
    }

    @Test
    fun `should throw ExistingResourceException when user with CPF already exists`() {
        val payload = RegisterRequestModel("teste", "12345678911", "OAKODOab@1", "OAKODOab@1")
        every { userRepository.findByCpf(any()) } returns Optional.of(anUser())

        assertThrows<ExistingResourceException>("Um usuário com esse CPF já está cadastrado") {
            userService.create(payload)
        }
    }

    @Test
    fun `should throw ExistingResourceException when user with email already exists`() {
        val payload = RegisterRequestModel("teste", "12345678911", "OAKODOab@1gmail.com", "OAKODOab@1")
        every { userRepository.findByCpf(any()) } returns Optional.empty()
        every { userRepository.findByEmail(any()) } returns Optional.of(anUser())

        assertThrows<ExistingResourceException>("Um usuário com esse email já está cadastrado") {
            userService.create(payload)
        }
    }

    @Test
    fun `should throw InvalidPasswordException when payload is invalid`() {
        val payload = RegisterRequestModel("teste", "12345678911", "OAKODOab@1", "OAKODOab@1")

        every { userRepository.findByCpf(any()) } returns Optional.empty()
        every { userRepository.findByEmail(any()) } returns Optional.empty()
        every { userValidator.validatePayload(any()) } throws ValidationException("Senha inválida")

        assertThrows<ValidationException>("Senha inválida") { userService.create(payload) }
    }

    private fun anUser() = User(
        id = UUID.randomUUID(),
        email = "teste@gmail.com",
        name = "teste",
        phoneNumber = "51999999999",
        cpf = "12345678911",
        isDeleted = false,
    )
}